@@ -302,9 +302,16 @@ static char *get_basedir(void)
 static char *get_epidir(void)
 {
 	char *basedir, *epidir;
+	struct stat s;
+
 	basedir = get_basedir();
 	xasprintf(&epidir, "%s/epigrams", basedir);
 	free(basedir);
+	if (!OPT_GIVEN(TFORTUNE, BASEDIR) && stat(epidir, &s) < 0) {
+		free(epidir);
+		epidir = xstrdup(DATADIR "/tfortunes/epigrams");
+		INFO_LOG("falling back to system-wide epidir %s\n", epidir);
+	}
 	return epidir;
 }
 
@@ -863,23 +870,20 @@ static int com_stats(void)
 	struct linhash_item *itemp;
 	unsigned num_epi_files, num_epis, num_unique_tags, num_x = 0;
 	long unsigned num_tags = 0;
-	char *xdir, *lh_stats;
+	char *xdir;
 	struct regfile_iter *riter;
-	bool verbose = OPT_GIVEN(STATS, VERBOSE);
 
 	tagtab = hash_tags(&num_epi_files, &num_epis);
 	for (
 		liter = linhash_iterator_new(tagtab, NULL, false);
 		(itemp = linhash_iterator_item(liter));
 		linhash_iterator_next(liter)
-	)
+	) {
+		free((char *)itemp->key);
 		num_tags += (long unsigned)itemp->object;
+	}
 	num_unique_tags = linhash_num_items(tagtab);
 	linhash_iterator_free(liter);
-	if (verbose)
-		lh_stats = linhash_statistics(tagtab);
-	linhash_free(tagtab);
-
 	xdir = get_xdir();
 	for (
 		regfile_iter_new(xdir, &riter);
@@ -895,15 +899,17 @@ static int com_stats(void)
 	printf("number of tags..................... %5lu\n", num_tags);
 	printf("number of unique tags.............. %5u\n", num_unique_tags);
 	printf("average number of epigrams per file %8.02f\n",
-		(float)num_epis / num_epi_files);
-	printf("average number of tags per epigram. %8.02f\n",
-		(float)num_tags / num_epis);
-	printf("average number of tag recurrence... %8.02f\n",
-		(float)num_tags / num_unique_tags);
-	if (verbose) {
+		num_epi_files > 0?  (float)num_epis / num_epi_files : 0);
+	printf("average number of tags per epigram. %8.02f\n", num_epis > 0?
+		(float)num_tags / num_epis : 0);
+	printf("average number of tag recurrence... %8.02f\n", num_unique_tags > 0?
+		(float)num_tags / num_unique_tags : 0);
+	if (OPT_GIVEN(STATS, VERBOSE)) {
+		char *lh_stats = linhash_statistics(tagtab);
 		printf("\nlinear hashing statistics:\n%s\n", lh_stats);
 		free(lh_stats);
 	}
+	linhash_free(tagtab);
 	return 1;
 }
 EXPORT_CMD_HANDLER(stats);
